"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startFoundationSimulationServer = exports.createFoundationSimulationServer = void 0;
const express_1 = __importDefault(require("express"));
const cors_1 = __importDefault(require("cors"));
const fdir_1 = require("fdir");
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const lodash_1 = require("lodash");
const openapi_backend_1 = __importDefault(require("openapi-backend"));
const ajv_formats_1 = __importDefault(require("ajv-formats"));
const index_1 = require("./store/index");
const routeTemplate_1 = require("./routeTemplate");
function createFoundationSimulationServer({ port = 9000, serveJsonFiles, openapi, extendStore, extendRouter, }) {
    return () => {
        let app = (0, express_1.default)();
        app.use((0, cors_1.default)());
        app.use(express_1.default.json());
        app.use(express_1.default.urlencoded({ extended: false }));
        let simulationStore = (0, index_1.createSimulationStore)(extendStore);
        app.use((req, res, next) => {
            simulationStore.store.dispatch(simulationStore.actions.simulationLog({
                method: req.method,
                url: req.url,
                query: req.query,
                body: req.body,
            }));
            next();
        });
        if (serveJsonFiles) {
            const jsonFiles = new fdir_1.fdir()
                .filter((path, _isDirectory) => path.endsWith(".json"))
                .withDirs()
                .withRelativePaths()
                .crawl(serveJsonFiles)
                .sync();
            if (jsonFiles.length > 0) {
                const simulationRoutes = [];
                for (let jsonFile of jsonFiles) {
                    const route = `/${jsonFile.slice(0, jsonFile.length - 5)}`;
                    const filename = node_path_1.default.join(serveJsonFiles, jsonFile);
                    app.get(route, (_req, res) => {
                        res.setHeader("content-type", "application/json");
                        node_fs_1.default.createReadStream(filename).pipe(res);
                    });
                    simulationRoutes.push(simulationStore.schema.simulationRoutes.add({
                        [`get:${route}`]: {
                            type: "JSON",
                            url: route,
                            method: "get",
                            calls: 0,
                            defaultCode: 200,
                            responses: [200],
                        },
                    }));
                }
                simulationStore.store.dispatch(simulationStore.actions.batchUpdater(simulationRoutes));
            }
        }
        if (extendRouter) {
            extendRouter(app, simulationStore);
        }
        if (openapi) {
            for (let spec of openapi) {
                let { document, handlers, apiRoot } = spec;
                let mergedOAS = Array.isArray(document)
                    ? mergeDocumentArray(document)
                    : document;
                let api = new openapi_backend_1.default({
                    definition: mergedOAS,
                    apiRoot,
                    customizeAjv: (ajv) => {
                        (0, ajv_formats_1.default)(ajv);
                        return ajv;
                    },
                });
                // register your framework specific request handlers here
                let handlerObjectRegistration = (handlerEntries, prefix) => {
                    for (let [key, handler] of Object.entries(handlerEntries)) {
                        if (typeof handler === "object") {
                            handlerObjectRegistration(handler, key);
                        }
                        else {
                            api.register(`${prefix ? `${prefix}/` : ""}${key}`, handler);
                        }
                    }
                };
                handlerObjectRegistration(handlers(simulationStore));
                api.register({
                    validationFail: (c, req, res) => res.status(400).json({ err: c.validation.errors }),
                    // if route not in API, continue to next API or `app.all('*')` fallback
                    notFound: (c, req, res, next) => next(),
                    notImplemented: (c, req, res) => {
                        let { status, mock } = c.api.mockResponseForOperation(
                        // the route validates this exists and throws if it does not
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        c.operation.operationId);
                        return res.status(status).json(mock);
                    },
                });
                // initalize the backend
                api.init().then((init) => {
                    const router = init.router;
                    const operations = router.getOperations();
                    const simulationRoutes = operations.reduce((routes, operation) => {
                        var _a;
                        const url = `${router.apiRoot}${operation.path}`;
                        routes[`${operation.method}:${url}`] = {
                            type: "OpenAPI",
                            url,
                            method: operation.method,
                            calls: 0,
                            defaultCode: 200,
                            responses: Object.keys((_a = operation.responses) !== null && _a !== void 0 ? _a : {}).map((key) => parseInt(key)),
                        };
                        return routes;
                    }, {});
                    simulationStore.store.dispatch(simulationStore.actions.batchUpdater([
                        simulationStore.schema.simulationRoutes.add(simulationRoutes),
                    ]));
                    return init;
                });
                app.use((req, res, next) => {
                    const routeId = `${req.method.toLowerCase()}:${req.path}`;
                    const routeMetadata = simulationStore.schema.simulationRoutes.selectById(simulationStore.store.getState(), {
                        id: routeId,
                    });
                    return api.handleRequest(req, req, res, next, routeMetadata);
                });
            }
        }
        app.get("/", (req, res) => {
            let routes = simulationStore.schema.simulationRoutes.selectTableAsList(simulationStore.store.getState());
            let logs = simulationStore.schema.simulationLogs.selectTableAsList(simulationStore.store.getState());
            if (routes.length === 0) {
                res.sendStatus(404);
            }
            else {
                res.status(200).send((0, routeTemplate_1.generateRoutesHTML)(routes, logs));
            }
        });
        app.post("/", (req, res) => {
            const formValue = req.body;
            const entries = {};
            for (let [key, value] of Object.entries(formValue)) {
                entries[key] = { defaultCode: parseInt(value) };
            }
            simulationStore.store.dispatch(simulationStore.actions.batchUpdater([
                simulationStore.schema.simulationRoutes.patch(entries),
            ]));
            res.redirect("/");
        });
        // if no extendRouter routes or openapi routes handle this, return 404
        app.all("*", (req, res) => res.status(404).json({ error: "not found" }));
        return {
            listen: async (portOverride, callback) => {
                let listeningPort = portOverride !== null && portOverride !== void 0 ? portOverride : port;
                let server = app.listen(listeningPort, callback);
                if (!server.listening) {
                    await new Promise((resolve) => {
                        server.once("listening", resolve);
                    });
                }
                return {
                    server,
                    simulationStore,
                    ensureClose: async () => {
                        await new Promise((resolve) => {
                            server.once("close", resolve);
                            server.closeIdleConnections();
                            server.closeAllConnections();
                            server.close();
                        });
                        // it takes a bit to close the server, but there is not method
                        //  that will cleanly await that process
                        await new Promise((resolve) => setTimeout(resolve, 20));
                    },
                };
            },
        };
    };
}
exports.createFoundationSimulationServer = createFoundationSimulationServer;
const mergeDocumentArray = (documents) => {
    let document = (0, lodash_1.merge)({}, ...documents);
    return document;
};
async function startFoundationSimulationServer(arg) {
    let simulation = createFoundationSimulationServer(arg)();
    return simulation.listen();
}
exports.startFoundationSimulationServer = startFoundationSimulationServer;
//# sourceMappingURL=index.js.map